/*
 * (c) Copyright 2000-2012 spring saarlouis
 * All Rights Reserved.
 *
 * contact info@spring.de for more information
 */
package de.spring.mobile;

import java.lang.ref.WeakReference;

import android.content.Context;
import android.graphics.Point;
import android.os.Build;
import android.view.WindowManager;
import android.widget.VideoView;

/**
 * A sample StreamAdapter implementation
 * Implementation of an adapter for the {@link VideoView} with standard 
 * android api.
 * 
 * @author <a href="mailto:support@spring.de">spring GmbH &amp; Co.KG</a>
 */
public class VideoViewAdapter implements StreamAdapter {

	private WeakReference<VideoView> videoViewWeakReference;
	
	/**
	 * Create the adapter with the given {@link VideoView}.
	 * 
	 * @param videoView The video view
	 * 
	 * @throws NullPointerException If the parameter <code>videoView</code> is <code>null</code>.
	 */
	public VideoViewAdapter(VideoView videoView) {
		if (videoView == null) {
			throw new NullPointerException("videoView may not be null");
		}
		videoViewWeakReference = new WeakReference<VideoView>(videoView);
	}

	/**
	 * Returns the meta object.
	 * 
	 * @see de.spring.mobile.StreamAdapter#getMeta()
	 */
	@Override
	public Meta getMeta() {
		return new Meta() {

			/**
			 * Returns the string <code>android.widget.VideoView</code>.
			 * 
			 * @see de.spring.mobile.StreamAdapter.Meta#getPlayerName()
			 * 
			 * @return The player name <code>android.widget.VideoView</code>.
			 */
			public String getPlayerName() {
				return "android.widget.VideoView";
			}

			/**
			 * Returns the version of the operating system.
			 * 
			 * @see Build.VERSION
			 * @see de.spring.mobile.StreamAdapter.Meta#getPlayerVersion()
			 * 
			 * @return The system version.
			 */
			public String getPlayerVersion() {
				return Build.VERSION.RELEASE;
			}

			/**
			 * Returns the screen height.
			 * 
			 * @see WindowManager#getDefaultDisplay(){@link #getWidth()}
			 * @see de.spring.mobile.StreamAdapter.Meta#getScreenHeight()
			 */
			public int getScreenHeight() {
				VideoView videoView = videoViewWeakReference.get();
				if (videoView != null) {
					Point point = new Point();
					(((WindowManager)videoViewWeakReference.get().getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay()).getSize(point);
					return point.y;
				}
				else return 0;
			}

			/**
			 * Returns the screen width.
			 * 
			 * @see WindowManager#getDefaultDisplay(){@link #getHeight()}
			 * @see de.spring.mobile.StreamAdapter.Meta#getScreenWidth()
			 */
			public int getScreenWidth() {
				VideoView videoView = videoViewWeakReference.get();
				if (videoView != null) {
					Point point = new Point();
					(((WindowManager)videoViewWeakReference.get().getContext().getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay()).getSize(point);
					return point.x;
				}else return 0;
			}

		};
	}
	
	/**
	 * Returns the position by calling the method {@link VideoView#getCurrentPosition()}.
	 * 
	 * @see de.spring.mobile.StreamAdapter#getPosition()
	 * @see VideoView#getCurrentPosition()
	 * 
	 * @return The position in seconds.
	 */
	@Override
	public int getPosition() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) {
			return (int)Math.round(videoView.getCurrentPosition()/1000.0);
		}
		return 0;
	}

	/**
	 * Returns the duration by calling the method {@link VideoView#getDuration()}.
	 * 
	 * @see de.spring.mobile.StreamAdapter#getDuration()
	 * @see VideoView#getCurrentPosition()
	 * 
	 * @return The duration in seconds.
	 */
	@Override
	public int getDuration() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return (int)Math.round(videoView.getDuration()/1000.0);
		else return 0;
	}

	/**
	 * Returns the width of the {@link VideoView}
	 * 
	 * @see VideoView#getWidth()
	 * @see de.spring.mobile.StreamAdapter#getWidth()
	 * 
	 * @return The width
	 */
	@Override
	public int getWidth() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return videoView.getWidth();
		else return 0;
	}

	/**
	 * Returns the height of the {@link VideoView}
	 * 
	 * @see VideoView#getHeight()
	 * @see de.spring.mobile.StreamAdapter#getHeight()
	 * 
	 * @return The height
	 */
	@Override
	public int getHeight() {
		VideoView videoView = videoViewWeakReference.get();
		if (videoView != null) return videoView.getHeight();
		else return 0;
	}

	/**
	 * Returns if video is streaming.
	 */
	@Override
	public boolean isCasting() {
		return false;
	}

}
